﻿/*! \file
 *  \brief Заголовочный файл с функциями для преобразования типов
 */

#ifndef TYPES_CONVERTER_H
#define TYPES_CONVERTER_H

#include "elcore50.h"
/*!
 *  \fn void float16_to_float32(uint16* src, float* dst, int size)
 *  \brief Преобразование вектора типа float16 в float
 *  \param[in] src входные данные типа uint16
 *  \param[out] dst выходные данные типа float
 *  \param[in] size размер массива
 */
extern void float16_to_float32(uint16* src, float* dst, int size);
/*!
 *  \fn void float32_to_float16(float* src, uint16* dst, int size)
 *  \brief Преобразование вектора типа float в float16
 *  \param[in] src входные данные типа float
 *  \param[out] dst выходные данные типа uint16
 *  \param[in] size размер массива
 */
extern void float32_to_float16(float* src, uint16* dst, int size);
/*!
 *  \fn void float32_to_fract16(float *src, int16 *dst, const int size)
 *  \brief Преобразование вектора типа float в fractional16
 *  \param[in] src входные данные типа float
 *  \param[out] dst выходные данные типа int16
 *  \param[in] size размер входных данных
 */
void float32_to_fract16(float *src, int16 *dst, const int size);
/*!
 *  \fn void fract16_to_float32(int16 *src, float *dst, const int size)
 *  \brief Преобразование вектора типа fractional16 в float
 *  \param[in] src входные данные типа int16
 *  \param[out] dst выходные данные типа float
 *  \param[in] size размер входных данных
 */
void fract16_to_float32(int16 *src, float *dst, const int size);

#endif